import copy


def find_actors(actor_list, letter):
    """
    Find actors whose last names start with the given letter. Actor names are
    tuples of (last, first). This function does not modify the actor_list. It
    returns a new list of actors.

    :param actor_list: a list of actors
    :param letter: a letter
    :return: a list of actors
    >>> test_list = [('Freeman', 'Morgan'), ('Streep', 'Meryl'), ('Hanks', 'Tom'), ('Bullock', 'Sandra')]
    >>> find_actors(test_list, 'B')
    [('Bullock', 'Sandra')]
    >>> test_list
    [('Freeman', 'Morgan'), ('Streep', 'Meryl'), ('Hanks', 'Tom'), ('Bullock', 'Sandra')]
    """
    pass


def find_actors2(actor_list, name, letter):
    """
    Find actors whose last names or first names start with the given letter.
    Actor names are tuples of (last, first). This function does not modify
    the actor_list. It returns a new list of actors.

    :param actor_list: a list of actors
    :param name: either 'last' or 'first'
    :param letter: a letter
    :return: a list of actors
    >>> test_list = [('Freeman', 'Morgan'), ('Streep', 'Meryl'), ('Hanks', 'Tom'), ('Bullock', 'Sandra')]
    >>> find_actors2(test_list, 'last', 'S')
    [('Streep', 'Meryl')]
    >>> find_actors2(test_list, 'first', 'S')
    [('Bullock', 'Sandra')]
    >>> find_actors2(test_list, 'last', 'Z')
    []
    >>> test_list
    [('Freeman', 'Morgan'), ('Streep', 'Meryl'), ('Hanks', 'Tom'), ('Bullock', 'Sandra')]
    """
    pass


def find_movies(movie_data, actor):
    """
    Find all the movies an actor has starred in. movie_data is a dictionary that
    maps movie titles to a list of the actors that starred in the movie; actors
    are listed as tuples of (last_name, first_name). This function does not modify
    the movie_data. It returns a new dictionary.

    :param movie_data: a dictionary that maps movie titles to a list of actors
    :param actor: an actor tuple
    :return: a new dictionary of movies
    >>> test_movies = {
    ...     'Suffragette': [('Mulligan', 'Carey'), ('Duff', 'Anne-Marie'),
    ...                     ('Bonham   Carter', 'Helena'), ('Streep', 'Meryl')],
    ...     'Into the Woods': [('Kendrick', 'Anna'), ('Streep', 'Meryl'),
    ...                        ('Pine', 'Chris'), ('Blunt', 'Emily'),
    ...                        ('Cordon', 'James'), ('Depp', 'Johnny')],
    ...     'Wonder Woman': [('Godot', 'Gal'), ('Pine', 'Chris'), ('Wright', 'Robin'),
    ...                      ('Davis', 'Lucy')]}
    >>> find_movies(test_movies, ('Streep', 'Meryl'))
    {'Suffragette': [('Mulligan', 'Carey'), ('Duff', 'Anne-Marie'), ('Bonham   Carter', 'Helena'), ('Streep', 'Meryl')], 'Into the Woods': [('Kendrick', 'Anna'), ('Streep', 'Meryl'), ('Pine', 'Chris'), ('Blunt', 'Emily'), ('Cordon', 'James'), ('Depp', 'Johnny')]}
    >>> find_movies(test_movies, ('Pine', 'Chris'))
    {'Into the Woods': [('Kendrick', 'Anna'), ('Streep', 'Meryl'), ('Pine', 'Chris'), ('Blunt', 'Emily'), ('Cordon', 'James'), ('Depp', 'Johnny')], 'Wonder Woman': [('Godot', 'Gal'), ('Pine', 'Chris'), ('Wright', 'Robin'), ('Davis', 'Lucy')]}
    >>> test_movies
    {'Suffragette': [('Mulligan', 'Carey'), ('Duff', 'Anne-Marie'), ('Bonham   Carter', 'Helena'), ('Streep', 'Meryl')], 'Into the Woods': [('Kendrick', 'Anna'), ('Streep', 'Meryl'), ('Pine', 'Chris'), ('Blunt', 'Emily'), ('Cordon', 'James'), ('Depp', 'Johnny')], 'Wonder Woman': [('Godot', 'Gal'), ('Pine', 'Chris'), ('Wright', 'Robin'), ('Davis', 'Lucy')]}
    """
    pass


def leading_actor(movie_data):
    """
    Create a dictionary that lists leading actors for a movie. The input to the
    function is a movie data dictionary that maps movie titles to a list of the
    actors that starred in the movie; actors are listed as tuples of
    (last_name, first_name). This function does not modify the movie_data.
    It returns a new dictionary.

    :param movie_data: a dictionary that maps movie titles to a list of actors
    :return: a new dictionary with just the leading actor for each movie
    >>> test_movies = {
    ...     'Suffragette': [('Mulligan', 'Carey'), ('Duff', 'Anne-Marie'),
    ...                     ('Bonham   Carter', 'Helena'), ('Streep', 'Meryl')],
    ...     'Into the Woods': [('Kendrick', 'Anna'), ('Streep', 'Meryl'),
    ...                        ('Pine', 'Chris'), ('Blunt', 'Emily'),
    ...                        ('Cordon', 'James'), ('Depp', 'Johnny')],
    ...     'Wonder Woman': [('Godot', 'Gal'), ('Pine', 'Chris'), ('Wright', 'Robin'),
    ...                      ('Davis', 'Lucy')]}
    >>> leading_actor(test_movies)
    {'Suffragette': [('Mulligan', 'Carey')], 'Into the Woods': [('Kendrick', 'Anna')], 'Wonder Woman': [('Godot', 'Gal')]}
    >>> test_movies
    {'Suffragette': [('Mulligan', 'Carey'), ('Duff', 'Anne-Marie'), ('Bonham   Carter', 'Helena'), ('Streep', 'Meryl')], 'Into the Woods': [('Kendrick', 'Anna'), ('Streep', 'Meryl'), ('Pine', 'Chris'), ('Blunt', 'Emily'), ('Cordon', 'James'), ('Depp', 'Johnny')], 'Wonder Woman': [('Godot', 'Gal'), ('Pine', 'Chris'), ('Wright', 'Robin'), ('Davis', 'Lucy')]}
    """
    pass


def currently_streaming(movie_list):
    """
    Find the movies that are currently streaming. Each movie in the list is a
    dictionary with the following structure:

    { title: string, genre: string, streaming: boolean }

    A movie is currently streaming if the value of 'streaming' is True.
    This function does not modify movie_list and returns a new list.

    :param movie_list: a list of movies
    :return: a new list with only the movies that are currently streaming
    >>> test_movies = [{ 'title': 'The Batman', 'genre': 'action', 'streaming': False},
    ...                { 'title': 'Dog', 'genre': 'comedy', 'streaming': False},
    ...                { 'title': 'Death on the Nile', 'genre': 'mystery', 'streaming': False},
    ...                { 'title': 'Turning Red', 'genre': 'comedy', 'streaming': True},
    ...                { 'title': 'The Adam Project', 'genre': 'sci-fi', 'streaming': True},
    ...                { 'title': 'Spider-Man: No Way Home', 'genre': 'fantasy', 'streaming': False}]
    >>> currently_streaming(test_movies)
    [{'title': 'Turning Red', 'genre': 'comedy', 'streaming': True}, {'title': 'The Adam Project', 'genre': 'sci-fi', 'streaming': True}]
    >>> test_movies
    [{'title': 'The Batman', 'genre': 'action', 'streaming': False}, {'title': 'Dog', 'genre': 'comedy', 'streaming': False}, {'title': 'Death on the Nile', 'genre': 'mystery', 'streaming': False}, {'title': 'Turning Red', 'genre': 'comedy', 'streaming': True}, {'title': 'The Adam Project', 'genre': 'sci-fi', 'streaming': True}, {'title': 'Spider-Man: No Way Home', 'genre': 'fantasy', 'streaming': False}]
    """
    pass


def all_streaming(movie_list):
    """
    Make all movies available for streaming. Each movie in the list is a
    dictionary with the following structure:

    { title: string, genre: string, streaming: boolean }

    This function does not modify movie_list and returns a new list where
    streaming is True for all movies.

    :param movie_list: a list of movies
    :return: a new list with all the movies set to be streaming
    >>> test_movies = [{ 'title': 'The Batman', 'genre': 'action', 'streaming': False},
    ...                { 'title': 'Dog', 'genre': 'comedy', 'streaming': False},
    ...                { 'title': 'Death on the Nile', 'genre': 'mystery', 'streaming': False},
    ...                { 'title': 'Turning Red', 'genre': 'comedy', 'streaming': True},
    ...                { 'title': 'The Adam Project', 'genre': 'sci-fi', 'streaming': True},
    ...                { 'title': 'Spider-Man: No Way Home', 'genre': 'fantasy', 'streaming': False}]
    >>> all_streaming(test_movies)
    [{'title': 'The Batman', 'genre': 'action', 'streaming': True}, {'title': 'Dog', 'genre': 'comedy', 'streaming': True}, {'title': 'Death on the Nile', 'genre': 'mystery', 'streaming': True}, {'title': 'Turning Red', 'genre': 'comedy', 'streaming': True}, {'title': 'The Adam Project', 'genre': 'sci-fi', 'streaming': True}, {'title': 'Spider-Man: No Way Home', 'genre': 'fantasy', 'streaming': True}]
    >>> test_movies
    [{'title': 'The Batman', 'genre': 'action', 'streaming': False}, {'title': 'Dog', 'genre': 'comedy', 'streaming': False}, {'title': 'Death on the Nile', 'genre': 'mystery', 'streaming': False}, {'title': 'Turning Red', 'genre': 'comedy', 'streaming': True}, {'title': 'The Adam Project', 'genre': 'sci-fi', 'streaming': True}, {'title': 'Spider-Man: No Way Home', 'genre': 'fantasy', 'streaming': False}]
    """
    pass


def convert_prices(price_data, exchange_rate):
    """
    Convert all the prices in the price_data dictionary to a new currency based
    on the exchange rate. The structure of price_data is:

    item -> price

    Where the key is an item (a string) and the value is a price (a float). The
    prices are in dollars.

    :param price_data: a dictionary that maps items to prices
    :param exchange_rate: an exchange rate from the original currency into a new currency
    :return a dictionary that maps items to prices in the new currency
    >>> test_prices = { 'jeans': 35.71, 'leather jacket': 120.22, 'floral dress': 55.58 }
    >>> euro_prices = convert_prices(test_prices, 0.91)
    >>> euro_prices
    {'jeans': 32.5, 'leather jacket': 109.4, 'floral dress': 50.58}
    >>> peso_prices = convert_prices(test_prices, 20.72)
    >>> peso_prices
    {'jeans': 739.91, 'leather jacket': 2490.96, 'floral dress': 1151.62}
    """
    for item, price in price_data.items():
        price_data[item] = round(price * exchange_rate, 2)

    return price_data

